/*
 * @(#) $Id: TableWM2UIListenerClient.java 2696 2008-09-08 10:25:08Z webok $
 *
 * Copyright 2004/2005 by SPARDAT Sparkassen-Datendienst Ges.m.b.H.,
 * A-1110 Wien, Geiselbergstr.21-25.
 * All rights reserved.
 *
 */
package at.spardat.xma.mdl.table;

import org.eclipse.swt.widgets.TableItem;

/**
 * This interface can be implemented to enhance the TableWM to UI behauvior for each cell.
 * Implement the method {@link TableWM2UIListenerClient#model2UIEvent(TableRow, TableItem, int, int)}.
 * This method is always called after a TableWM cell is written to an SWT TableItem index.
 * Additional model2UI behauvior, like setting of fonts and colours dynamically can be implemented there.
 * Register your class with {@link ITableWMClient#setTableWM2UIListener(TableWM2UIListenerClient)} .setTableWM2UIListener(TableWM2UIListenerClient).
 *
 * @author s3460
 * @since 2.0.4
 */
public interface TableWM2UIListenerClient {

    /**
     * This method is always called after a TableWM cell is written to the SWT TableItem index.
     * You can implement this method for additional model2UI behauvior, like setting of fonts and colours.
     *
     * @param tr the table model's TableRow
     * @param ti the SWT widget's TableItem
     * @param modelColumn use this index to access cells from the TableRow
     * @param swtColumn use this index to access the TableItem
     * @since 2.0.4
     * @author s3460
     */
    public void model2UIEvent(TableRow tr, TableItem ti, int modelColumn, int swtColumn);
}
